
/*------------------------------------------------------------------
* class InputPanel.
*
* author(s):  Diana Garroway
* date:       27-Oct-2003
*
* Description:  This class handels coordinating the two polygons
*
* -------------------------------------------------------------------*/

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Iterator;

public class InputPanel extends JPanel
                        implements ActionListener{

    //******************************************************************
    //  Member Variables
    //

    private GeometryCanvas m_Polygon1;
    private GeometryCanvas m_Polygon2;
    private JPanel m_CanvasPanel;
    private JPanel m_ButtonPanel;
    private JButton m_bStep;
    private JButton m_bStepBack;
    private JButton m_bResetAll;


    private boolean m_Polygon1Ready;
    private boolean m_Polygon2Ready;

    public final int StartSTATE = 0;
    public final int ReadySTATE = 1;
    public final int CardinalitySTATE = 2;
    public final int CorrespondingSTATE = 3;
    public final int TriangulatedSTATE = 4;
    public final int CenterComputedSTATE = 5;
    public final int WeightsFoundSTATE = 6;
    public final int DrawIsoTriangulationSTATE = 7;

    boolean[] m_StateMemory;
    int m_iState = 0;
    final int m_iStateSize = 8;

    //******************************************************************
    //  Constructors
    //
    public InputPanel(){
        m_Polygon1 = new GeometryCanvas( "1", this );
        m_Polygon2 = new GeometryCanvas( "2", this );
        m_CanvasPanel = new JPanel(new GridLayout(0,2));
        m_ButtonPanel = new JPanel();
        m_ButtonPanel.setLayout( new BoxLayout(m_ButtonPanel, BoxLayout.LINE_AXIS));

        m_CanvasPanel.add(m_Polygon1);
        m_CanvasPanel.add(m_Polygon2);

        m_StateMemory = new boolean[m_iStateSize];
        for( int i = 0; i < m_iStateSize; i++){
            m_StateMemory[i] = false;
        }

        m_bStep = new JButton("Step");
        m_bStep.setEnabled(false);
        m_bStep.setActionCommand("Step");
        m_bStep.addActionListener(this);

        m_bStepBack = new JButton("Step Back");
        m_bStepBack.setEnabled(false);
        m_bStepBack.setActionCommand("Step Back");
        m_bStepBack.addActionListener(this);


        m_bResetAll = new JButton("Reset All");
        m_bResetAll.setEnabled(true);
        m_bResetAll.setActionCommand("Reset");
        m_bResetAll.addActionListener(this);

        m_ButtonPanel.add(m_bStep);
        m_ButtonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        m_ButtonPanel.add(m_bStepBack);
        m_ButtonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        m_ButtonPanel.add(m_bResetAll);

        this.setLayout(new BorderLayout( 8, 8));
        this.add(m_CanvasPanel, BorderLayout.CENTER);
        this.add(m_ButtonPanel, BorderLayout.SOUTH);

        SimpleMorph.m_statusWindow.SendMessage(" Please enter 2 simple polygons in the above spaces. \n" +
                                                "   --> You can enter polygons by clicking on the white space.\n" +
                                                "   --> Finish the polygon by clicking on the starting vertex.");
    }

    //******************************************************************
    //  Data Accessors
    //
    public int GetState(){
        return m_iState;
    }

    //******************************************************************
    //  Maintenance Methods
    //
   
    //******************************************************************
    //  Main Methods
    //
    public void actionPerformed(ActionEvent e) {
        if ("Step".equals(e.getActionCommand())) {
            //ready to triangulate
            if (m_iState == ReadySTATE){
                m_iState++;
                SimpleMorph.m_statusWindow.SendMessage("Step1: Adjust the cardinaltiy of the Polygons so they each have equal number of vertices.\n" +
                                                    "   --> Click on the step button to continue the algorithm");
                m_bStep.setEnabled(false);
                if ( !m_StateMemory[m_iState-1]){
                    if ( m_Polygon1.GetInputPolygon().GetSize() > m_Polygon2.GetInputPolygon().GetSize() )
                        m_Polygon2.GetInputPolygon().AdjustSize( m_Polygon1.GetInputPolygon().GetSize() );
                    else if (m_Polygon1.GetInputPolygon().GetSize() < m_Polygon2.GetInputPolygon().GetSize() )
                        m_Polygon1.GetInputPolygon().AdjustSize( m_Polygon2.GetInputPolygon().GetSize() );
                    m_StateMemory[m_iState-1] = true;
                }
                m_Polygon1.repaint();
                m_Polygon2.repaint();
                m_bStep.setEnabled(true);
                m_bStepBack.setEnabled(true);
            }
            else if (m_iState == CardinalitySTATE){
                m_iState++;
                SimpleMorph.m_statusWindow.SendMessage("Step2: Pick a vertex from each Polygon to indicate the correspondance between the two.\n" +
                                                    "   --> Click on the step button to continue the algorithm");
                m_bStep.setEnabled(false);
                 m_Polygon1.repaint();
                m_Polygon2.repaint();
                m_bStep.setEnabled(true);
            }
            else if (m_iState == CorrespondingSTATE){
                m_iState++;
                SimpleMorph.m_statusWindow.SendMessage("Step3: Triangulate each Polygon and find the dual tree.\n" +
                                                    "   --> Click on the step button to continue the algorithm");
                m_bStep.setEnabled(false);
                if ( !m_StateMemory[m_iState-1]){
                    m_Polygon1.GetInputPolygon().Triangulate();
                    m_Polygon2.GetInputPolygon().Triangulate();
                    m_StateMemory[m_iState-1] = true;
                }
                m_Polygon1.repaint();
                m_Polygon2.repaint();
                m_bStep.setEnabled(true);
            }
            //triangulated
            else if (m_iState == TriangulatedSTATE){
                m_iState++;
                SimpleMorph.m_statusWindow.SendMessage("Step4: Find a center vertex of the tree.\n" +
                                                    "   --> Click on the step button to continue the algorithm");
                m_bStep.setEnabled(false);
                m_Polygon1.repaint();
                m_Polygon2.repaint();
                m_bStep.setEnabled(true);
            }

            else if (m_iState == CenterComputedSTATE){

                m_iState++;
                SimpleMorph.m_statusWindow.SendMessage("Step5: Calcuate the weight of each vertex\n" +
                                                    "   --> Click on the step button to continue the algorithm");
                m_bStep.setEnabled(false);
                m_Polygon1.repaint();
                m_Polygon2.repaint();
                m_bStep.setEnabled(true);
            }
            else if (m_iState == WeightsFoundSTATE){

                m_iState++;
                SimpleMorph.m_statusWindow.SendMessage("Step6: Set the vertex weights to be the maximum of the corresponding vertices.\n" +
                                                    "   --> Click on the step button to continue the algorithm");
                m_bStep.setEnabled(false);

                if ( !m_StateMemory[m_iState-1]){
                    ArrayList l_sequence1 = m_Polygon1.GetInputPolygon().GetVertexWeights();
                    ArrayList l_sequence2 = m_Polygon2.GetInputPolygon().GetVertexWeights();

                    ArrayList l_isoSequence = new ArrayList();
                    Iterator j = l_sequence2.iterator();
                    for (Iterator i = l_sequence1.iterator(); i.hasNext(); ) {
                        Integer l_weight1 = (Integer) i.next();
                        Integer l_weight2 = (Integer) j.next();
                        if ( l_weight1.compareTo(l_weight2) > 0 )
                            l_isoSequence.add (l_weight1);
                        else
                            l_isoSequence.add (l_weight2);
                    }
                    m_Polygon1.GetInputPolygon().SetIsoSequence(l_isoSequence);
                    m_Polygon2.GetInputPolygon().SetIsoSequence(l_isoSequence);
                    m_StateMemory[m_iState-1] = true;
                }
                m_Polygon1.repaint();
                m_Polygon2.repaint();
                m_bStep.setEnabled(true);
            }
            else if (m_iState == DrawIsoTriangulationSTATE){
                m_iState++;
                SimpleMorph.m_statusWindow.SendMessage("Step7: Draw the isomorphic triangulation according to the spiderweb connectivity of the verticies.\n" +
                                                    "   --> The Algorthim is now DONE!");
                m_bStep.setEnabled(false);

                if ( !m_StateMemory[m_iState-1]){
                    m_Polygon1.GetInputPolygon().CreateIsoTriangulation();
                    m_Polygon2.GetInputPolygon().CreateIsoTriangulation();
                    m_StateMemory[m_iState-1] = true;
                }
                m_Polygon1.repaint();
                m_Polygon2.repaint();

            }
        }
        else if ("Reset".equals(e.getActionCommand())){
            m_iState =0;
            m_Polygon1.GetInputPolygon().Clear();
            m_Polygon2.GetInputPolygon().Clear();
            for(int i = 0; i< m_iStateSize; i++){
                m_StateMemory[i] = false;
            }
            m_bStep.setEnabled(false);
            m_bStepBack.setEnabled(false);
           SimpleMorph.m_statusWindow.SendMessage(" Please enter 2 simple polygons in the above spaces. \n" +
                                                "   --> You can enter polygons by clicking on the white space.\n" +
                                                "   --> Finish the polygon by clicking on the starting vertex.");
        }
        else if ("Step Back".equals(e.getActionCommand())){

            m_iState--;
            if (m_iState == 1)
                m_bStepBack.setEnabled(false);
            else
            {
                m_iState--;
                m_bStep.setEnabled(true);
                m_bStep.doClick(0);
            }

        }
        this.repaint();
    }


    public void SetPolygonState( String in_string, boolean in_state ){
        if ( in_string.equals("1") )
          m_Polygon1Ready = in_state;
        else
          m_Polygon2Ready = in_state;

        if (m_Polygon1Ready && m_Polygon2Ready){
           m_bStep.setEnabled(true);
           m_iState = ReadySTATE;
           SimpleMorph.m_statusWindow.SendMessage("You may now Step through the Isomorphic Triangulation algorithm!\n" +
                                                    "   --> Do this by clicking the step button!");
        }
        else {
            m_bStep.setEnabled(false);
            m_iState = 0;
        }
    }
}
