
/*------------------------------------------------------------------
* class StatusWindow.
*
* author(s):  Diana Garroway
* date:       24-Oct-2003
*
* Description:  This class manages the status window of the gui.  It is
*               used to inform the user of possible actions and of results
*               of those actions
* -------------------------------------------------------------------*/

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.text.*;
import java.awt.*;

public class StatusWindow extends JPanel{
    //******************************************************************
    //  Member Variables
    //
    private JTextPane m_textPane;
    private JScrollPane m_scrollPane;

    //******************************************************************
    //  Constructors
    //
    public StatusWindow (){
        super( new BorderLayout() );
        m_textPane = new JTextPane();
        m_textPane.setEditable(false);
        Style def = StyleContext.getDefaultStyleContext().
                                        getStyle(StyleContext.DEFAULT_STYLE);

        Style regular = m_textPane.addStyle("regular", def);
        StyleConstants.setFontFamily(def, "SansSerif");

        Style s = m_textPane.addStyle("Error", regular);
        StyleConstants.setItalic(s, true);
        StyleConstants.setForeground(s,Color.red);
        m_scrollPane = new JScrollPane(m_textPane);

        StyledDocument l_Doc = m_textPane.getStyledDocument();
        SendMessage( "Welcome to the Simple Polygon Morph Applet");

        Border l_canvasOutline = BorderFactory.createLineBorder(Color.black);
        Border l_title = BorderFactory.createTitledBorder(
		       l_canvasOutline, "Status Window");

        this.setBorder(l_title);
        this.setBackground(Color.white);
        this.add(m_scrollPane);

    }

    //******************************************************************
    //  Data Accessors
    //
   
    //******************************************************************
    //  Maintenance Methods
    //

    //******************************************************************
    //  Main Methods
    //
    public void AddMessage(String in_String){
        StyledDocument l_Doc = m_textPane.getStyledDocument();
        try{
            l_Doc.insertString( l_Doc.getLength(), "\n" + in_String, m_textPane.getStyle("regular"));
        } catch (BadLocationException ble) {
            System.err.println("Couldn't insert initial text.");
        }
    }

     public void SendMessage(String in_String){
        StyledDocument l_Doc = m_textPane.getStyledDocument();
        try{
            ClearWindow();
            l_Doc.insertString(0, "\n" + in_String, m_textPane.getStyle("regular"));
        } catch (BadLocationException ble) {
            System.err.println("Couldn't insert initial text.");
        }
    }

    public void SendError(String in_String){
        StyledDocument l_Doc = m_textPane.getStyledDocument();
        try{
            ClearWindow();
            l_Doc.insertString(0, "\n" + in_String, m_textPane.getStyle("Error"));
        } catch (BadLocationException ble) {
            System.err.println("Couldn't insert initial text.");
        }
    }

    public void ClearWindow(){
        StyledDocument l_Doc = m_textPane.getStyledDocument();
        try {
            l_Doc.remove( 0, l_Doc.getLength());
        } catch (BadLocationException e) {
            System.err.println("Couldn't clear the Status Window Text.");
        }
    }
}
