
/*------------------------------------------------------------------
* class cDualTree.
*
* author(s):  Diana Garroway
* date:       10-Dec-2003
*
* Description: 
*
* -------------------------------------------------------------------*/

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import java.awt.*;

public class cDualTree {
    //******************************************************************
    //  Member Variables
    //
    ArrayList m_VertexList;
    ArrayList m_EdgeList;
    cVertex m_Center;

    //******************************************************************
    //  Constructors
    //
    public cDualTree(){
        m_VertexList = new ArrayList();
        m_EdgeList = new ArrayList();
        m_Center = null;
    }

    //******************************************************************
    //  Data Accessors
    //
    public List GetVertexList( ){
        return m_VertexList;
    }
    public List GetEdgeList( ){
        return m_EdgeList;
    }

    //******************************************************************
    //  Maintenance Methods
    //
    public void AddVertex( cVertex in_vertex){
        in_vertex.SetIndex( m_VertexList.size() );
        m_VertexList.add(in_vertex);
    }
    public void AddEdge( cEdge in_edge){
        m_EdgeList.add(in_edge);
    }
    public void clear(){
        m_VertexList.clear();
        m_EdgeList.clear();
        m_Center = null;
    }

    //******************************************************************
    //  Main Methods
    //
    public cVertex CalcalateCenter(){

        int l_minDistance = 10000;
        cVertex l_Center = (cVertex) m_VertexList.get(0);
        for (Iterator i = m_VertexList.iterator(); i.hasNext();){
            cVertex l_current = (cVertex) i.next();
            int l_currentDis = FindFurthest(l_current);
            if ( l_currentDis < l_minDistance ){
               l_Center = l_current;
               l_minDistance = l_currentDis;
            }
        }
        m_Center = l_Center;
        return l_Center;
    }

    public void AssignWeights(){
       m_Center.SetDepth( 1 );

       for (Iterator i = m_Center.GetDualVertices().iterator(); i.hasNext();){
            cVertex l_current = (cVertex)i.next();
            SetDepths(l_current, m_Center, 1);
       }
    }

    public void Draw (Graphics in_Graphic){
        in_Graphic.setColor(Color.blue);
        for (Iterator i = m_VertexList.iterator(); i.hasNext(); ){
                cVertex l_current = (cVertex) i.next();
                ArrayList l_neighbours = (ArrayList) l_current.GetDualVertices();
                for (Iterator j = l_neighbours.iterator(); j.hasNext();){
                    cVertex l_neighbour = (cVertex) j.next();
                    cEdge l_edge = new cEdge( l_neighbour, l_current);
                    l_edge.Draw( in_Graphic );
                }
            }
        for (Iterator i = m_VertexList.iterator(); i.hasNext();){
            cVertex l_current = (cVertex) i.next();
            l_current.DrawSmall(in_Graphic);
        }
    }

    //******************************************************************
    //  Helper Methods
    //
    public int FindFurthest( cVertex in_vertex ){
        int l_MaxSoFar = 0;
        for (Iterator i = in_vertex.GetDualVertices().iterator(); i.hasNext();){
           cVertex l_neighbour = (cVertex)i.next();
           int l_distance = FindFurthestFrom( l_neighbour, in_vertex ) + 1;
           if (l_distance > l_MaxSoFar)
               l_MaxSoFar = l_distance;
        }
        return l_MaxSoFar;
    }

    public int FindFurthestFrom( cVertex in_home, cVertex in_from ){
        int l_MaxSoFar = 0;
        for (Iterator i = in_home.GetDualVertices().iterator(); i.hasNext();){
           cVertex l_neighbour = (cVertex)i.next();
           if (l_neighbour != in_from){
                int l_distance = FindFurthestFrom( l_neighbour, in_home ) + 1;
                if (l_distance > l_MaxSoFar)
                    l_MaxSoFar = l_distance;
           }
        }
        return l_MaxSoFar;
    }

   public void SetDepths(cVertex in_current, cVertex in_from, int oldDepth){
       in_current.SetDepth( oldDepth + 1);
       for (Iterator i = in_current.GetDualVertices().iterator(); i.hasNext();){
           cVertex l_neighbour = (cVertex)i.next();
           if (l_neighbour != in_from){
               SetDepths(l_neighbour, in_current, in_current.GetDepth());
           }
       }
   }
}
