/*------------------------------------------------------------------
* class cGraph
*
* author(s):  Diana Garroway
* date:       Oct 20, 2003
*
* This class takes care of graph/Polygon related properties...
*
* -------------------------------------------------------------------*/

import java.util.*;
import java.util.List;
import java.awt.*;

public class cSimplePolygon {

    //******************************************************************
     //  Member Variables
     //

     private ArrayList m_VertexList;
     private ArrayList m_DiagonalList;
     private cPointi m_CentrePoint;
     private cDualTree m_Dual;
     private ArrayList m_InsideDiagonals;
     private boolean m_Finished;
     private String m_name;
     private InputPanel m_inputCanvas;
     private ArrayList m_IsoSequence;
     final static int EPSILON = 4;

      //******************************************************************
      //  Constructors
      //

     public cSimplePolygon( String in_name, InputPanel in_inputCanvas ) {
       m_VertexList   = new ArrayList();
       m_DiagonalList = new ArrayList();
       m_Dual = new cDualTree();
       m_IsoSequence = new ArrayList();
       m_InsideDiagonals = new ArrayList();
       m_CentrePoint  = null;
       m_Finished = false;
       m_name = in_name;
       m_inputCanvas = in_inputCanvas;
     }

      //******************************************************************
      //  Data Accessors
      //
      public ArrayList GetVertexList(){
          return m_VertexList;
      }
      public void SetVertexList( ArrayList in_VertexList ){
          m_VertexList = in_VertexList;
      }
      public ArrayList GetDiagonalList(){
          return m_DiagonalList;
      }
      public void SetDiagonalList( ArrayList in_DiagonalList ){
          m_DiagonalList = in_DiagonalList;
      }
      public ArrayList GetIsoSequence(){
          return m_IsoSequence;
      }
      public void SetIsoSequence( ArrayList in_isoSequence ){
          m_IsoSequence = in_isoSequence;
          for (Iterator i = m_VertexList.iterator(); i.hasNext(); ) {
              cVertex l_current = (cVertex) i.next();
              Integer l_depth = (Integer) in_isoSequence.get(l_current.GetIndex());
              l_current.SetOriginalDepth( l_current.GetDepth() );
              l_current.SetDepth( l_depth.intValue() );
          }
      }
      public ArrayList GetVertexWeights(){
          ArrayList l_vertexWeights = new ArrayList();
          for (Iterator i = m_VertexList.iterator(); i.hasNext(); ) {
            cVertex l_current = (cVertex) i.next();
            l_vertexWeights.add( new Integer(l_current.GetDepth()) );
          }
          return l_vertexWeights;
      }

      public cPointi GetCentrePoint(){
          return m_CentrePoint;
      }
      public void SetCentrePoint( cPointi in_CentrePoint ){
          m_CentrePoint = in_CentrePoint;
      }
      public String GetName(){
          return m_name;
      }
      public void SetName( String in_name ){
          m_name = in_name;
      }
      public boolean Finished(){
          return m_Finished;
      }
      public void SetFinished( boolean in_Value ){
          m_Finished = in_Value;
          if (m_Finished){
              int sum = 0;
              int l_vertexCount = 1;
              cVertex l_head = (cVertex)m_VertexList.get(0);
              for( CircularLLIterator i = new CircularLLIterator( m_VertexList, 1 ); l_vertexCount < m_VertexList.size();){
                  cVertex l_v0 = (cVertex)i.next();
                  cVertex l_v1 = (cVertex)i.next();
                  sum += l_head.GetPoint().Area2( l_head.GetPoint(), l_v0.GetPoint(), l_v1.GetPoint());
                  i.previous();
                  l_vertexCount++;
              }

              if (sum < 0){
                  ArrayList l_tempList = new ArrayList();
                  l_tempList.add( m_VertexList.get(0) );
                  int count = 1;
                  //reverse the list if the vertices were given in ccw order
                  for (ListIterator i = m_VertexList.listIterator( m_VertexList.size() ); i.hasPrevious(); ){
                     cVertex l_current = (cVertex) i.previous();
                     l_current.SetIndex( count );
                     l_tempList.add( l_current );
                     count++;
                  }
                  m_VertexList = l_tempList;
                  m_VertexList.remove(  m_VertexList.size()-1 );
                  cVertex l_first = (cVertex)m_VertexList.get(0);
                  l_first.SetIndex(0);
              }
              if (!VerifySimplness()){
                  SimpleMorph.m_statusWindow.SendError("Polygon " + m_name + " was not simple."   +
                                                        "    --> Please start again (no edge may cross)");
                  Clear();
              }
              else
                  m_inputCanvas.SetPolygonState( GetName(), true );
          }
      }
      public int GetSize(){
          return m_VertexList.size();
      }

     //********************************************************************
     //  Maintenance Methods
     //
     public void AddVertex( cVertex in_Vertex ){
         if (!m_Finished){
            //compare to the first vertex
            if ( m_VertexList.size() > 0 ){
                cVertex l_firstPoint = (cVertex) m_VertexList.get(0);
                if (  l_firstPoint.GetPoint().Dist(in_Vertex.GetPoint()) > 15)
                      m_VertexList.add( in_Vertex );
                else
                    SetFinished(true);
            }
            else
                m_VertexList.add( in_Vertex );
         }
     }

    public void AddVertex( cVertex in_Vertex, boolean in_finish ){
         if (!m_Finished){
            //compare to the first vertex
            if ( m_VertexList.size() > 0 ){
                if (in_finish){
                    cVertex l_firstPoint = (cVertex) m_VertexList.get(0);
                    if (  l_firstPoint.GetPoint().Dist(in_Vertex.GetPoint()) > 8)
                          m_VertexList.add( in_Vertex );

                    else
                        SetFinished(true);
                }
                else
                    m_VertexList.add(in_Vertex);
            }
            else
                m_VertexList.add( in_Vertex );
         }
     }

     public void Clear(){
        m_VertexList.clear();
        m_DiagonalList.clear();
        m_IsoSequence.clear();
        m_InsideDiagonals.clear();
        m_Dual.clear();
        m_CentrePoint = null;
        m_Finished = false;
        m_inputCanvas.SetPolygonState( GetName(), false );

     }

    public void AdjustSize (int in_size){
        int l_numberToAdd = in_size - m_VertexList.size();
        for (CircularLLIterator i = new CircularLLIterator(m_VertexList, 0); l_numberToAdd > 0; ) {
            cVertex l_1 = (cVertex) i.next();
            cVertex l_2 = (cVertex) i.next();
            i.previous();
            cVertex l_new = new cVertex( l_1.GetPoint().MidPoint(l_2.GetPoint()));
            l_new.SetExtra(true);
            i.add(l_new);
            i.next();
            l_numberToAdd--;
        }
        //re-adjust the index of the vertices
        int l_count = 0;
        for (Iterator i = m_VertexList.iterator(); i.hasNext(); ) {
            cVertex l_vertex = (cVertex) i.next();
            l_vertex.SetIndex(l_count);
            l_count++;
        }
    }
     //*********************************************************************
     //  Main Methods
     //

    /*---------------------------------------------------------------------
     * This algorithm is from the book "Computational Geometry in C" by
     * Joseph O'Rourke.  It has been modified to also calculate a center
     * vertex and the "milestone depth" of each vertex.  These concepts
     * can be found in the paper:
     *      Guarenteed Intersection Free Polygon Morphing
     *          Craig Gotsman, Vitaly Surazhsky
     */
    public boolean Triangulate()
    {
      cVertex l_v0, l_v1, l_v2, l_v3, l_v4;
      cEdge l_diagonal;
      List  l_vertexListCopy = (List) m_VertexList.clone();
      int l_vertexCount = 0;

      Stack l_earOrder = new Stack();

      if (!m_Finished){
             SimpleMorph.m_statusWindow.SendMessage("You must finish the polygon before triangulating. \n"
                                                    + "   --> This means the last vertex must equal the first vertex.");
          return false;
      }

      CalculateCenter0fGravity();
      EarInit( );
      l_v2 = FindFurthestEar( l_vertexListCopy );
      while ( l_vertexListCopy.size() > 3 && ( l_v2 != null ) ){

       CircularLLIterator i = new CircularLLIterator( l_vertexListCopy, l_vertexListCopy.indexOf( l_v2 ) );
       i.next();
       l_v3 = (cVertex) i.next(); l_v4 = (cVertex) i.next();
       i.previous(); i.previous();  i.previous();// move the iterator back to v2
       l_v1 = (cVertex) i.previous(); l_v0 = (cVertex) i.previous();
       i.next(); i.next(); i.next();

       /* Update earity of diagonal endpoints */
       l_v1.SetIsAnEar( Diagonal( l_v0, l_v3 ) );
       l_v3.SetIsAnEar( Diagonal( l_v1, l_v4 ) );

       l_v2.SetNeighbours( l_v1, l_v3 );
       l_earOrder.push(l_v2);
       cPointi l_CentrePoint = l_v0.GetPoint().Centroid3( l_v1.GetPoint(), l_v2.GetPoint(), l_v3.GetPoint());
       cVertex l_dualVertex = new cVertex( l_CentrePoint );
       l_v1.AddDualVertex(l_dualVertex);
       l_v2.AddDualVertex(l_dualVertex);
       l_v3.AddDualVertex(l_dualVertex);
       m_Dual.AddVertex( l_dualVertex );

       /* (v1,v3) is a diagonal */
       l_diagonal = new cEdge(l_v1, l_v3);
       l_diagonal.SetNeighbour(l_dualVertex);
       m_DiagonalList.add(l_diagonal);

       /* add edge to dual if required */
       for (Iterator j=m_DiagonalList.iterator(); j.hasNext();){
           cEdge l_current = (cEdge)j.next();
           if (l_current.MatchEdgePoints(l_v1, l_v2)
               || l_current.MatchEdgePoints(l_v2, l_v3)){
               l_dualVertex.AddDualVertex( l_current.GetNeighbour());
               l_current.GetNeighbour().AddDualVertex(l_dualVertex);
           }
       }

       /* Cut off the ear v2 */
       l_vertexListCopy.remove( l_v2 );
       l_v2 = FindFurthestEar( l_vertexListCopy);
      }

      if ( ( l_vertexListCopy.size() >3 ) && ( l_v2 == null ) ) {
            SimpleMorph.m_statusWindow.SendError( "Polygon " + m_name + " was not simple: cannot triangulate" +
                                                      "\n  --> Please try again...no edges can cross!");
            Clear();
            return false;
      }

      CircularLLIterator l_centerTri = new CircularLLIterator(l_vertexListCopy, 0);
      l_v0 = (cVertex) l_centerTri.next();
      l_v1 = (cVertex) l_centerTri.next();
      l_v2 = (cVertex) l_centerTri.next();

      //Add the last point to our tree
      m_CentrePoint = l_v0.GetPoint().Centroid3( l_v0.GetPoint(), l_v1.GetPoint(), l_v2.GetPoint());
      cVertex l_dualVertex = new cVertex( m_CentrePoint );
      l_v2.AddDualVertex(l_dualVertex);
      l_v0.AddDualVertex(l_dualVertex);
      l_v1.AddDualVertex(l_dualVertex);
      m_Dual.AddVertex( l_dualVertex );

      /* add edge to dual if required */
       for (Iterator j=m_DiagonalList.iterator(); j.hasNext();){
           cEdge l_current = (cEdge)j.next();
           if (l_current.MatchEdgePoints(l_v0, l_v1)
               || l_current.MatchEdgePoints(l_v1, l_v2)
               || l_current.MatchEdgePoints(l_v0, l_v2)){
               l_dualVertex.AddDualVertex( l_current.GetNeighbour());
               l_current.GetNeighbour().AddDualVertex(l_dualVertex);
           }
       }
      cVertex l_centerVertex = m_Dual.CalcalateCenter();
      m_CentrePoint = l_centerVertex.GetPoint();

      for (Iterator i = m_VertexList.iterator(); i.hasNext();){
        cVertex l_current = (cVertex) i.next();
        for (Iterator j = l_current.GetDualVertices().iterator(); j.hasNext();){
            cVertex l_dual = (cVertex) j.next();
            if (l_dual == l_centerVertex){
                l_current.SetOriginal(true);
            }
        }
      }

      m_Dual.AssignWeights();

      for (Iterator i = m_VertexList.iterator(); i.hasNext();){
          cVertex l_current = (cVertex)i.next();
          ArrayList l_DualVerticies = (ArrayList)l_current.GetDualVertices();
          int l_depth = 10000;
          for (Iterator j = l_DualVerticies.iterator(); j.hasNext();){
              cVertex l_neighbour = (cVertex) j.next();
              if (l_depth > l_neighbour.GetDepth()){
                  l_depth = l_neighbour.GetDepth();
              }
          }
          l_current.SetDepth(l_depth);
          l_current.SetOriginalDepth( l_current.GetDepth() );
      }


      return true;
    }


    public boolean VerifySimplness(){
       cVertex l_c0, l_c1, l_a, l_b;
       int l_vertexCountOutside = 0;
       int l_vertexCountInside = 0;

       for (CircularLLIterator i = new CircularLLIterator(m_VertexList, 0); l_vertexCountOutside < m_VertexList.size(); ) {
           l_a = (cVertex) i.next();
           l_b = (cVertex) i.next();
           for (CircularLLIterator j = new CircularLLIterator(m_VertexList, 0); l_vertexCountInside < m_VertexList.size(); ) {
             l_c0 = (cVertex) i.next();
             l_c1 = (cVertex) i.next();
             i.previous();
             /* Skip edges incident to a or b */
             if (    ( l_c0 != l_a ) && ( l_c1 != l_a )
                     && ( l_c0 != l_b ) && ( l_c1 != l_b )
                     && l_a.GetPoint().Intersect( l_a.GetPoint(), l_b.GetPoint(), l_c0.GetPoint(), l_c1.GetPoint() )
                     )
                return false;

             l_vertexCountInside++;
           }
           i.previous();
           l_vertexCountInside = 0;
           l_vertexCountOutside++;
       }
     return true;
    }


    public void CreateIsoTriangulation(){

        int l_topLevel = 0;
        int[] l_tempDepths = new int[m_VertexList.size()];
        int l_count = 0;
        ArrayList l_projectedOrder = new ArrayList();

        for (Iterator i = m_VertexList.iterator(); i.hasNext(); ){
            cVertex l_current = (cVertex)i.next();
            l_tempDepths[l_count] = l_current.GetDepth();
            if (l_current.GetDepth() > l_topLevel)
                l_topLevel = l_current.GetDepth();
            l_count++;
        }

        for (int j = l_topLevel; j > 1; j--){
           int l_vertexCount = 0;
           for (CircularLLIterator i = new CircularLLIterator(m_VertexList, 0); l_vertexCount < m_VertexList.size(); ) {
               cVertex l_previous = (cVertex) i.previous(); i.next();
               cVertex l_current = (cVertex) i.next();
               cVertex l_next = (cVertex) i.next();  i.previous();
               cVertex l_new;
               l_current.SetProjectedAtLevel(false);
               if ( (l_tempDepths[l_vertexCount] == j) && !(l_current.GetProjected()) ){
                   l_previous = FindPreviousNotProjected(l_previous);
                   l_next = FindNextNotProjected(l_next);
                   cPointi l_temp = l_current.GetPoint().FindPoints( l_previous.GetPoint(), l_next.GetPoint(), EPSILON*(l_current.GetDepth()-j +1) );
                   l_new = new cVertex(l_temp);
                   l_current.AddInsideVertex(l_new);
                   l_tempDepths[l_vertexCount] -= 1;
               }
               l_vertexCount++;
           }

           l_vertexCount = 0;
           cSimplePolygon l_levelPolygon = new cSimplePolygon("temp", m_inputCanvas);
           ArrayList l_tempVertexList = (ArrayList) m_VertexList.clone();
           for(Iterator k = l_tempVertexList.iterator(); k.hasNext();){
                cVertex l_temp = (cVertex)k.next();
                if (l_temp.GetInsideVertices().size() > 0){
                    l_temp = (cVertex)l_temp.GetInsideVertices().get(0);
                }
                l_temp.SetIndex(l_vertexCount);
                l_levelPolygon.AddVertex(l_temp, false);
                l_vertexCount++;
           }
          l_vertexCount = 0;
          if (l_levelPolygon.GetSize() > 2){
           // Determine which verticies can be projected at this level
           for(CircularLLIterator i = new CircularLLIterator(m_VertexList, 0); l_vertexCount < m_VertexList.size(); ) {
               cVertex l_previousTop = (cVertex) i.previous(); i.next();
               cVertex l_currentTop = (cVertex) i.next();
               cVertex l_nextTop = (cVertex) i.next();  i.previous();
               cVertex l_previous = null, l_current = null, l_next = null;

               if ( !l_currentTop.GetOriginal()
                     && (l_currentTop.GetInsideVertices().size() > 0)
                     && !l_currentTop.GetProjected()
                     && !PreviousProjectedAtLevel( l_vertexCount ))
               {
                   //Set up current
                   l_current = (cVertex) l_currentTop.GetInsideVertices().get(0);

                   //Set up the previous vertex to check
                   int l_index = l_levelPolygon.GetVertexList().indexOf(l_current);
                   if (l_index == 0)
                        l_index = l_levelPolygon.GetSize()-1;
                   else
                        l_index--;
                   for(CircularLLIterator k = new CircularLLIterator (l_levelPolygon.GetVertexList(), l_index );
                       ((l_previous == null) && (l_previousTop != l_nextTop)); )  {
                       if ( !l_previousTop.GetProjected() ){
                        if (l_previousTop.GetInsideVertices().size() > 0)
                            l_previous = (cVertex) l_previousTop.GetInsideVertices().get(0);
                        else
                            l_previous = l_previousTop;
                       }
                       else
                           l_previousTop = (cVertex) k.previous();
                   }

                   //Set up the next vertex
                   l_index = l_levelPolygon.GetVertexList().indexOf( l_current );
                   if (l_index == l_levelPolygon.GetSize()-1)
                        l_index = 0;
                   else
                        l_index++;

                   for(CircularLLIterator k = new CircularLLIterator (l_levelPolygon.GetVertexList(), l_index );
                       ((l_next == null) && (l_previousTop != l_nextTop)); )  {
                       if ( !l_nextTop.GetProjected() ){
                        if (l_nextTop.GetInsideVertices().size() > 0)
                            l_next = (cVertex) l_nextTop.GetInsideVertices().get(0);
                        else
                            l_next = l_nextTop;
                       }
                       else
                           l_nextTop = (cVertex) k.next();
                   }

                   //Check to see if the diagonal exists, if l_previous is not null (we have not already projected the previous vertex
                    if ( l_previous != null && l_next != null){
                        if ( l_levelPolygon.Diagonal( l_previous, l_next )){
                            l_current.SetPoint( l_currentTop.GetPoint().FindProjectedPoint( l_previous.GetPoint(), l_next.GetPoint() ) );
                            l_currentTop.SetProjected(true);
                            l_currentTop.SetProjectedAtLevel(true);
                            l_projectedOrder.add(0, l_currentTop);
                            l_previousTop = (cVertex)m_VertexList.get( l_levelPolygon.GetVertexList().indexOf( l_previous));
                            l_nextTop = (cVertex)m_VertexList.get( l_levelPolygon.GetVertexList().indexOf( l_next));
                            l_currentTop.SetProjectedNeighbours(l_previousTop, l_nextTop);

                        }
                    }
               }
               l_vertexCount++;
           }
          }

         // Continue to project vertecies that were projected on a previous level
           for(Iterator i = l_projectedOrder.iterator(); i.hasNext(); ) {
                cVertex l_currentTop = (cVertex) i.next();

                if ( l_currentTop.GetProjected() && !l_currentTop.GetProjectedAtLevel()){
                    ArrayList l_neighbours = (ArrayList) l_currentTop.GetProjectedNeighbours();

                    cVertex l_neighbour1 = (cVertex)l_neighbours.get(0);
                    cVertex l_neighbour2 = (cVertex)l_neighbours.get(1);

                    if (l_neighbour1.GetInsideVertices().size() > 0)
                        l_neighbour1 = (cVertex) l_neighbour1.GetInsideVertices().get(0);
                    if (l_neighbour2.GetInsideVertices().size() > 0)
                        l_neighbour2 = (cVertex) l_neighbour2.GetInsideVertices().get(0);

                    cVertex l_current = (cVertex)l_currentTop.GetInsideVertices().get(0);
                    cPointi l_temp = l_currentTop.GetPoint().FindProjectedPoint( l_neighbour1.GetPoint(), l_neighbour2.GetPoint() );
                    cVertex l_new = new cVertex(l_temp);
                    l_currentTop.AddInsideVertex(l_new);
                }
            }
         }
    }


    public void CalculateCenter0fGravity(){
            m_CentrePoint = new cPointi();
            for (Iterator i = m_VertexList.iterator(); i.hasNext(); ) {
                cVertex l_current = (cVertex) i.next();
                m_CentrePoint.x += l_current.GetPoint().x;
                m_CentrePoint.y += l_current.GetPoint().y;
            }
            m_CentrePoint.x = m_CentrePoint.x/m_VertexList.size();
            m_CentrePoint.y = m_CentrePoint.y/m_VertexList.size();
    }


     public void Draw(Graphics in_Graphic){
        for (Iterator i = m_VertexList.iterator(); i.hasNext(); ) {
               cVertex l_current = (cVertex) i.next();
               if ( l_current.GetIndex() != 0 ) {
                   cVertex l_previous = (cVertex) m_VertexList.get( l_current.GetIndex() -1 );
                   in_Graphic.drawLine(l_previous.GetPoint().x, l_previous.GetPoint().y,
                                       l_current.GetPoint().x, l_current.GetPoint().y);
               }
        }
        if (m_Finished){
            cVertex l_first = (cVertex) m_VertexList.get(0);
            cVertex l_last = (cVertex) m_VertexList.get( m_VertexList.size() -1 );
            in_Graphic.drawLine(l_first.GetPoint().x, l_first.GetPoint().y,
                                       l_last.GetPoint().x, l_last.GetPoint().y);
        }

        if ( m_inputCanvas.GetState() >= m_inputCanvas.TriangulatedSTATE
             && m_inputCanvas.GetState() <= m_inputCanvas.DrawIsoTriangulationSTATE
             && m_DiagonalList.size() > 0) {
            in_Graphic.setColor(Color.red);
            for (Iterator i = m_DiagonalList.iterator(); i.hasNext(); ){
                cEdge l_edge = (cEdge) i.next();
                l_edge.Draw( in_Graphic );
            }
            m_Dual.Draw(in_Graphic);
        }

         if (m_inputCanvas.GetState() > m_inputCanvas.DrawIsoTriangulationSTATE){

            int l_vertexCount = 0;
            in_Graphic.setColor(Color.green);
            for (CircularLLIterator i = new CircularLLIterator(m_VertexList, 0); l_vertexCount < m_VertexList.size(); ) {
                l_vertexCount++;
                cVertex l_current = (cVertex) i.next();
                cVertex l_next = (cVertex) i.next(); i.previous();

                l_current.DrawInsideEdges( in_Graphic, l_next);
            }

            cVertex l_v0 = null, l_v1 = null, l_v2 = null;
            int l_count = 0;
            for(Iterator i = m_VertexList.iterator(); i.hasNext();){
                cVertex l_current = (cVertex) i.next();
                if ( l_current.GetOriginal() ){
                    if (l_count == 0)
                        l_v0 = l_current;
                    else if (l_count == 1)
                        l_v1 = l_current;
                    else if (l_count == 2)
                        l_v2 = l_current;
                    l_count++;
                }
            }
            cPointi l_newCenter = l_v0.GetPoint().Centroid3(l_v0.GetPoint(), l_v1.GetPoint(), l_v2.GetPoint());
            cVertex l_newCenterVertex = new cVertex(l_newCenter);

            for (Iterator i = m_VertexList.iterator(); i.hasNext(); ){
                cVertex l_current = (cVertex) i.next();
                cEdge l_temp = new cEdge();
                if (l_current.GetInsideVertices().size() > 0){
                    l_temp.SetEndPts( (cVertex)l_current.GetInsideVertices().get(0), l_newCenterVertex);
                }
                else
                    l_temp.SetEndPts( l_current, l_newCenterVertex);
                l_temp.Draw(in_Graphic);
            }
            in_Graphic.setColor(Color.black);
            in_Graphic.fillOval( l_newCenter.x -2,  l_newCenter.y -2, 4, 4);


            for (Iterator i = m_VertexList.iterator(); i.hasNext();){
                cVertex l_current = (cVertex) i.next();
                l_current.DrawInsideVertices( in_Graphic );
            }
        }

        boolean l_fillVertices;
        if (m_inputCanvas.GetState() >= m_inputCanvas.WeightsFoundSTATE )
            l_fillVertices = false;
        else
            l_fillVertices = true;

        boolean l_maxDepth;
        if (m_inputCanvas.GetState() == m_inputCanvas.WeightsFoundSTATE )
            l_maxDepth = false;
        else
            l_maxDepth = true;

        boolean l_drawExtras;
        if (m_inputCanvas.GetState() < m_inputCanvas.CardinalitySTATE )
            l_drawExtras = false;
        else
            l_drawExtras = true;

        boolean l_drawExtrasAsSpecial;
        if (m_inputCanvas.GetState() == m_inputCanvas.CardinalitySTATE )
            l_drawExtrasAsSpecial = true;
        else
            l_drawExtrasAsSpecial = false;

        boolean l_drawFirst;
        if (m_inputCanvas.GetState() > m_inputCanvas.CardinalitySTATE )
            l_drawFirst = true;
        else
            l_drawFirst = false;

        for (Iterator i = m_VertexList.iterator(); i.hasNext(); ) {
               cVertex l_current = (cVertex) i.next();
               l_current.Draw( in_Graphic, l_fillVertices, l_drawExtrasAsSpecial, l_drawFirst, l_maxDepth, l_drawExtras );
        }

        if (m_inputCanvas.GetState() >= m_inputCanvas.CenterComputedSTATE
            && m_inputCanvas.GetState() <= m_inputCanvas.DrawIsoTriangulationSTATE
            && m_CentrePoint != null){
                in_Graphic.setColor(Color.black);
            if (m_inputCanvas.GetState() == m_inputCanvas.CenterComputedSTATE)
                in_Graphic.fillOval( m_CentrePoint.x -8,  m_CentrePoint.y -8, 16, 16);
            else
                in_Graphic.fillOval( m_CentrePoint.x -4,  m_CentrePoint.y -4, 8, 8);
        }
     }

    //*********************************************************************
    //  Helper Methods
    //

     protected void EarInit( )
     {
        cVertex l_v0, l_v1, l_v2;
        int l_vertexCount =0;

        for (CircularLLIterator i = new CircularLLIterator(m_VertexList, 0); l_vertexCount < m_VertexList.size(); ) {
            l_v0 = (cVertex) i.next();
            l_v1 = (cVertex) i.next();
            l_v2 = (cVertex) i.next();
            i.previous(); i.previous();
            l_v1.SetIsAnEar( Diagonal( l_v0, l_v2 ) );
            l_vertexCount++;
        }

     }

    protected cVertex FindFurthestEar( List in_VertexList ){

        int l_vertexCount = 0;
        double l_maxDist = 0;
        cVertex l_maxEar = null;

        for (CircularLLIterator i = new CircularLLIterator( in_VertexList, 0); l_vertexCount < in_VertexList.size(); ) {
            cVertex l_current = (cVertex) i.next();
            double l_dist = l_current.GetPoint().Dist( m_CentrePoint );
            if ( l_current.IsAnEar() && ( l_dist > l_maxDist ) ){
                l_maxDist = l_dist;
                l_maxEar = l_current;
            }
            l_vertexCount++;
        }
        return l_maxEar;
    }

    /*---------------------------------------------------------------------
     * Returns TRUE iff the diagonal (a,b) is strictly internal to the
     * polygon in the neighborhood of the a endpoint.
     */
    protected boolean InCone( cVertex in_a, cVertex in_b )
    {
        cVertex l_a0,l_a1;	/* a0,a,a1 are consecutive vertices. */

        CircularLLIterator l_iterator = new CircularLLIterator(m_VertexList, in_a.GetIndex());

        l_a0 = (cVertex) l_iterator.previous();  l_iterator.next(); l_iterator.next();
        l_a1 = (cVertex) l_iterator.next();

        /* If a is a convex vertex ... */
        if( in_a.GetPoint().LeftOn( in_a.GetPoint(), l_a1.GetPoint(), l_a0.GetPoint() ) )
      return in_a.GetPoint().Left( in_a.GetPoint(), in_b.GetPoint(), l_a0.GetPoint() )
        &&   in_b.GetPoint().Left( in_b.GetPoint(), in_a.GetPoint(), l_a1.GetPoint() );

        /* Else a is reflex: */
        return !(    in_a.GetPoint().LeftOn( in_a.GetPoint(), in_b.GetPoint(), l_a1.GetPoint() )
          && in_b.GetPoint().LeftOn( in_b.GetPoint(), in_a.GetPoint(), l_a0.GetPoint() ) );
    }


    /*---------------------------------------------------------------------
      *Returns true iff (a,b) is a proper internal *or* external
      *diagonal of P, *ignoring edges incident to a and b*.
      */
     public boolean Diagonalie( cVertex in_a, cVertex in_b )
     {
       cVertex l_c0, l_c1;
       int l_vertexCount = 0;

       /* For each edge (c,c1) of P */
       for (CircularLLIterator i = new CircularLLIterator(m_VertexList, 0); l_vertexCount < m_VertexList.size(); ) {
         l_c0 = (cVertex) i.next();
         l_c1 = (cVertex) i.next();
         i.previous();
         /* Skip edges incident to a or b */
         if (    ( l_c0 != in_a ) && ( l_c1 != in_a )
                 && ( l_c0 != in_b ) && ( l_c1 != in_b )
                 && in_a.GetPoint().Intersect( in_a.GetPoint(), in_b.GetPoint(), l_c0.GetPoint(), l_c1.GetPoint() )
                 )
         return false;

         l_vertexCount++;
       }
       return true;
     }

       /*---------------------------------------------------------------------
   *Returns TRUE iff (a,b) is a proper internal diagonal.
   */
    public boolean Diagonal( cVertex a, cVertex b )
    {
      return InCone( a, b ) && InCone( b, a ) && Diagonalie( a, b );
    }

   public cVertex FindPreviousNotProjected( cVertex in_previous  ){

       int l_index = m_VertexList.indexOf(in_previous);
       cVertex l_return = null;

       int l_vertexCount = 0;
       if ( in_previous.GetProjected() )
            for( CircularLLIterator k = new CircularLLIterator (m_VertexList, l_index );
               ( (l_return == null) && (l_vertexCount != (m_VertexList.size()- 2) ));){

                cVertex l_current = (cVertex) k.previous();
                if (! l_current.GetProjected()){
                    if (l_current.GetInsideVertices().size() > 0)
                      l_return = (cVertex) l_current.GetInsideVertices().get(0);
                    else
                      l_return = l_current;
                }
                l_vertexCount++;
            }
       else{
           if (in_previous.GetInsideVertices().size() > 0 ){
               l_return = (cVertex) in_previous.GetInsideVertices().get(0);
           }
           else
               l_return = in_previous;
       }
       return l_return;
   }

   public cVertex FindNextNotProjected( cVertex in_next  ){

       int l_index = m_VertexList.indexOf(in_next);
       cVertex l_return = null;

       int l_vertexCount = 0;
       if ( in_next.GetProjected() )
            for( CircularLLIterator k = new CircularLLIterator (m_VertexList, l_index );
               ( (l_return == null) && (l_vertexCount != (m_VertexList.size()- 2) ));){

                cVertex l_current = (cVertex) k.next();
                if (! l_current.GetProjected()){
                    if (l_current.GetInsideVertices().size() > 0)
                      l_return = (cVertex) l_current.GetInsideVertices().get(0);
                    else
                      l_return = l_current;
                }
                l_vertexCount++;
            }
       else {
           if (in_next.GetInsideVertices().size() > 0 ){
               l_return = (cVertex) in_next.GetInsideVertices().get(0);
           }
           else
               l_return = in_next;
       }

       return l_return;
   }


    public boolean PreviousProjectedAtLevel( int in_vertexIndex ){
        boolean l_continue = true;
        boolean l_returnVal = true;
        for(CircularLLIterator i = new CircularLLIterator (m_VertexList, in_vertexIndex ); l_continue == true;){
            cVertex l_current = (cVertex) i.previous();
            if ( l_current.GetProjected() && l_current.GetProjectedAtLevel() ){
                l_continue = false;
                l_returnVal = true;
            }
            else if ( !l_current.GetProjected() ){
                l_continue = false;
                l_returnVal = false;
            }
        }
        return l_returnVal;
    }



}
