/*------------------------------------------------------------------
* class cVertex.
*
* author(s):  Diana Garroway
* date:       Oct 20, 2003
*
* This class takes care of vertex related properties...
*   Depth = the depth from the center vertex that this vertex has in its
*           polygon
*   NeighbourList = tracks neighbours needed for different algorithms.
*   Index  = Its index in the polygon.
* -------------------------------------------------------------------*/

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import java.util.ListIterator;
import java.awt.*;

class cVertex {

  //******************************************************************
  //  Member Variables
  //

  private cPointi m_Pointi;
  private boolean m_bIsAEar;
  private cVertex m_neighbour1, m_neighbour2;
  private int m_iDepth;
  private int m_iOriginalDepth;
  private int m_iIndex;
  private boolean m_bExtra;
  private double m_dAngle;
  private ArrayList m_InsideVertices;
  private ArrayList m_DualVertices;
  private boolean m_bOriginal;  //will be true if the origanal depth is 1
  private boolean m_bProjectedAtLevel;
  private boolean m_bProjected;
  private ArrayList m_ProjectedNeighbours;
  private ArrayList m_DependentNeighbours;

   //******************************************************************
   //  Constructors
   //

  public cVertex() {
    m_Pointi = new cPointi();
    m_InsideVertices = new ArrayList();
    m_DualVertices = new ArrayList();
    m_bIsAEar = false;
    m_bExtra = false;
    m_neighbour1 = null;
    m_neighbour2 = null;
    m_iDepth = 1;
    m_iOriginalDepth = 1;
    m_iIndex = -1;
    m_dAngle = 0;
    m_bOriginal = false;
    m_bProjectedAtLevel = false;
    m_bProjected = false;
    m_ProjectedNeighbours = new ArrayList();
    m_DependentNeighbours = new ArrayList();
  }

  public cVertex(int i, int j, int in_index) {
    m_Pointi = new cPointi(i,j);
    m_InsideVertices = new ArrayList();
    m_DualVertices = new ArrayList();
    m_bIsAEar = false;
    m_bExtra = false;
    m_neighbour1 = null;
    m_neighbour2 = null;
    m_iDepth = 1;
    m_iOriginalDepth = 1;
    m_iIndex = in_index;
    m_dAngle = 0;
    m_bOriginal = false;
    m_bProjectedAtLevel = false;
    m_bProjected = false;
    m_ProjectedNeighbours = new ArrayList();
    m_DependentNeighbours = new ArrayList();
}

 public cVertex( cPointi in_Pt) {
    m_Pointi = in_Pt;
    m_InsideVertices = new ArrayList();
    m_DualVertices = new ArrayList();
    m_bIsAEar = false;
    m_bExtra = false;
    m_neighbour1 = null;
    m_neighbour2 = null;
    m_iDepth = 1;
    m_iOriginalDepth = 1;
    m_iIndex = -1;
    m_dAngle = 0;
    m_bOriginal = false;
    m_bProjectedAtLevel = false;
    m_bProjected = false;
    m_ProjectedNeighbours = new ArrayList();
    m_DependentNeighbours = new ArrayList();
}

   //******************************************************************
   //  Data Accessors
   //

   public boolean IsAnEar(){
      return m_bIsAEar;
   }

   public void SetIsAnEar( boolean in_bIsAnEar ){
      m_bIsAEar = in_bIsAnEar;
   }

   public void SetExtra (boolean in_bExtra){
       m_bExtra = in_bExtra;
   }

   public cPointi GetPoint(){
       return m_Pointi;
   }
   public void SetPoint( cPointi in_Point ){
       m_Pointi = in_Point;
   }

   public void AddInsideVertex (cVertex in_vertex){
       in_vertex.SetProjected( m_bProjected );
       in_vertex.SetProjectedAtLevel(m_bProjectedAtLevel);
       m_InsideVertices.add( 0, in_vertex );
   }

   public ArrayList GetInsideVertices( ){
       return m_InsideVertices;
   }
   public void AddDualVertex( cVertex in_vertex ){
       m_DualVertices.add(in_vertex);
   }
   public List GetDualVertices(){
       return m_DualVertices;
   }

   public void AddDependentNeighbour( cVertex in_vertex ){
       m_DependentNeighbours.add(in_vertex);
   }

   public double GetAngle(){
       return m_dAngle;
   }
   public void SetAngle( double in_angle ){
       m_dAngle = in_angle;
   }
   public int GetDepth(){
       return m_iDepth;
   }
   public void SetDepth( int in_depth ){
       m_iDepth = in_depth;
   }
   public int GetOriginalDepth(){
       return m_iOriginalDepth;
   }
   public void SetOriginalDepth( int in_depth ){
       m_iOriginalDepth = in_depth;
   }

   public boolean GetOriginal(){
      return m_bOriginal;
   }
   public void SetOriginal(boolean in_original){
      m_bOriginal = in_original;
   }

   public boolean GetProjected(){
       return m_bProjected;
   }
   public void SetProjected( boolean in_value ){
       m_bProjected = in_value;
       for (Iterator i = m_InsideVertices.iterator(); i.hasNext();){
            cVertex l_temp = (cVertex)i.next();
            l_temp.SetProjected(in_value);
       }
   }

   public boolean GetProjectedAtLevel(){
       return m_bProjectedAtLevel;
   }
   public void SetProjectedAtLevel( boolean in_Projected ){
       m_bProjectedAtLevel = in_Projected;
       for (Iterator i = m_InsideVertices.iterator(); i.hasNext();){
            cVertex l_temp = (cVertex)i.next();
            l_temp.SetProjectedAtLevel(in_Projected);
       }
   }

   public int GetIndex(){
       return m_iIndex;
   }
   public void SetIndex( int in_index ){
       m_iIndex = in_index;
   }

   public List GetNeighbours(){
       ArrayList l_list = new ArrayList();
       l_list.add( m_neighbour1 );
       l_list.add( m_neighbour2 );
       return l_list;
   }
   public void SetNeighbours( cVertex in_Neighbour1, cVertex in_Neighbour2 ){
       m_neighbour1 = in_Neighbour1;
       m_neighbour2 = in_Neighbour2;
   }

   public List GetProjectedNeighbours(){
       return m_ProjectedNeighbours;
   }
   public void SetProjectedNeighbours( cVertex in_vertex1, cVertex in_vertex2 ){
       m_ProjectedNeighbours.clear();
       m_ProjectedNeighbours.add(in_vertex1);
       m_ProjectedNeighbours.add(in_vertex2);
   }



    //********************************************************************
    //  Maintenance Methods
    //
    public void NotifyDependents(){
        for (Iterator i = m_DependentNeighbours.iterator(); i.hasNext();){
            cVertex l_temp = (cVertex)i.next();
            l_temp.SetNeighbours( m_neighbour1, m_neighbour2);
            l_temp.NotifyDependents();
        }
    }

    //********************************************************************
    //  Main Methods
    //
    public void Draw( Graphics in_Graphic, boolean in_drawFilled,
                        boolean in_drawExtraSpecial, boolean in_drawFirst, boolean in_drawMax, boolean in_drawExtras ){
        if (in_drawFilled){
             if (in_drawExtraSpecial && m_bExtra)
                in_Graphic.setColor(Color.green);
             else
                in_Graphic.setColor(Color.blue);

             if (m_iIndex == 0 && in_drawFirst)
                in_Graphic.setColor(Color.red);
            if (in_drawExtras || !m_bExtra)
                in_Graphic.fillOval(m_Pointi.x -8,  m_Pointi.y -8, 16, 16);
        }
        else{
            in_Graphic.setColor(Color.white);
            in_Graphic.fillOval(m_Pointi.x -8,  m_Pointi.y -8, 16, 16);
            in_Graphic.setColor(Color.blue);
            if (m_iIndex == 0)
                in_Graphic.setColor(Color.red);
            if (m_bOriginal)
                 in_Graphic.setColor(Color.black);
            in_Graphic.drawOval(m_Pointi.x -8,  m_Pointi.y -8, 16, 16);
            if (in_drawMax)
                in_Graphic.drawString( Integer.toString( m_iDepth), m_Pointi.x-4, m_Pointi.y+4);
            else
                in_Graphic.drawString( Integer.toString( m_iOriginalDepth), m_Pointi.x-4, m_Pointi.y+4);
        }
    }

     public void DrawSmall( Graphics in_Graphic ){
        in_Graphic.setColor(Color.red);
        in_Graphic.fillOval(m_Pointi.x -2,  m_Pointi.y -2, 4, 4);
    }

     public void DrawInsideVertices ( Graphics in_Graphic ){
        for (Iterator i = m_InsideVertices.iterator(); i.hasNext();){
            cVertex l_current = (cVertex) i.next();
            l_current.DrawSmall( in_Graphic );
        }
     }

     public void DrawInsideEdges ( Graphics in_Graphic, cVertex in_next){
        boolean l_firstTime = true;
        cEdge l_edge;
        int l_level = m_InsideVertices.size()-1;
        for (ListIterator i = m_InsideVertices.listIterator( m_InsideVertices.size() ); i.hasPrevious();){
            cVertex l_current = (cVertex) i.previous();
            in_Graphic.setColor(Color.green);
            if (l_firstTime){
                l_edge = new cEdge(l_current, this);
                l_edge.Draw(in_Graphic);
                l_firstTime = false;
            }
            else {
                i.next();
                cVertex l_previous = (cVertex)i.next();  i.previous(); i.previous();
                l_edge = new cEdge(l_current, l_previous);
                l_edge.Draw(in_Graphic);

            }
            if (in_next.GetInsideVertices().size()-1 >= l_level ){
                ArrayList l_tempList = in_next.GetInsideVertices();
                l_edge = new cEdge( l_current, (cVertex) l_tempList.get(l_level));
                l_edge.Draw(in_Graphic);
            }
            else {
                l_edge = new cEdge (l_current, in_next);
                l_edge.Draw(in_Graphic);
            }
            l_level--;
        }
        if (in_next.GetInsideVertices().size() > this.GetInsideVertices().size()){
                for (int j = GetInsideVertices().size(); j < in_next.GetInsideVertices().size(); j++ ){
                    ArrayList l_tempList = in_next.GetInsideVertices();
                    l_edge = new cEdge(this, (cVertex) l_tempList.get(j));
                    l_edge.Draw(in_Graphic);
                }
        }
     }
}









